@@ -12,6 +12,7 @@ import math
 from svg_ultralight.bounding_boxes.supports_bounds import SupportsBounds
 from svg_ultralight.strings import svg_matrix
 from svg_ultralight.transformations import mat_apply, mat_dot, new_transformation_matrix
+from svg_ultralight.unit_conversion import MeasurementArg, to_user_units
 
 _Matrix = tuple[float, float, float, float, float, float]
 
@@ -39,7 +40,7 @@ class HasBoundingBox(SupportsBounds):
         y = self.bbox.base_y
         x2 = x + self.bbox.base_width
         y2 = y + self.bbox.base_height
-        return (x, y), (x2, y), (x2, y2), (x, y2)
+        return (x, y), (x, y2), (x2, y2), (x2, y)
 
     def values(self) -> tuple[float, float, float, float]:
         """Get the values of the bounding box.
@@ -65,12 +66,28 @@ class HasBoundingBox(SupportsBounds):
 
         :return: four corners counter-clockwise starting at top left, transformed by
             self.transformation
+
+        These quadrilateral defined by these corners may not be axis aligned. This is
+        purely for determining the bounds of the transformed box.
         """
         c0, c1, c2, c3 = (
             mat_apply(self.bbox.transformation, c) for c in self._get_input_corners()
         )
         return c0, c1, c2, c3
 
+    @property
+    def corners(
+        self,
+    ) -> tuple[
+        tuple[float, float],
+        tuple[float, float],
+        tuple[float, float],
+        tuple[float, float],
+    ]:
+        """Get the corners of the bbox in the current state. CW from top left."""
+        x, y, x2, y2 = self.x, self.y, self.x2, self.y2
+        return ((x, y), (x, y2), (x2, y2), (x2, y))
+
     def transform(
         self,
         transformation: _Matrix | None = None,
@@ -106,7 +123,7 @@ class HasBoundingBox(SupportsBounds):
     def scale(self) -> tuple[float, float]:
         """Get scale of the bounding box.
 
-        :return: uniform scale of the bounding box
+        :return: x and y scale of the bounding box
 
         Use caution, the scale attribute can cause errors in intuition. Changing
         width or height will change the scale attribute, but not the x or y values.
@@ -301,13 +318,13 @@ class BoundingBox(HasBoundingBox):
     :param transformation: transformation matrix
 
     Functions that return a bounding box will return a BoundingBox instance. This
-    instance can be transformed (uniform scale and translate only). Transformations
-    will be combined and scored to be passed to new_element as a transform value.
+    instance can be transformed. Transformations will be combined and stored to be
+    passed to new_element as a transform value.
 
     Define the bbox with x=, y=, width=, height=
 
     Transform the BoundingBox by setting these variables. Each time you set x, cx,
-    x2, y, cy, y2, width, or height, private transformation value transformation
+    x2, y, cy, y2, width, or height, private transformation value `transformation`
     will be updated.
 
     The ultimate transformation can be accessed through ``.transform_string``.
@@ -316,7 +333,7 @@ class BoundingBox(HasBoundingBox):
         1. Get the bounding box of an svg element
         2. Update the bounding box x, y, width, and height
         3. Transform the original svg element with
-            update_element(elem, transform=bbox.transform_string)
+            svg_ultralight.transform_element(elem, bbox.transformation)
         4. The transformed element will lie in the transformed BoundingBox
 
     In addition to x, y, width, and height, x2 and y2 can be set to establish the
@@ -352,10 +369,10 @@ class BoundingBox(HasBoundingBox):
 
     def __init__(
         self,
-        x: float,
-        y: float,
-        width: float,
-        height: float,
+        x: MeasurementArg,
+        y: MeasurementArg,
+        width: MeasurementArg,
+        height: MeasurementArg,
         transformation: _Matrix = (1, 0, 0, 1, 0, 0),
     ) -> None:
         """Initialize a BoundingBox instance.
@@ -365,38 +382,68 @@ class BoundingBox(HasBoundingBox):
         :param width: width of the bounding box
         :param height: height of the bounding box
         """
-        self.base_x = x
-        self.base_y = y
-        self.base_width = width
-        self.base_height = height
+        self.base_x = to_user_units(x)
+        self.base_y = to_user_units(y)
+        self.base_width = to_user_units(width)
+        self.base_height = to_user_units(height)
         self.transformation = transformation
         self.bbox = self
 
-    def merge(self, *others: BoundingBox) -> BoundingBox:
+    def join(self, *others: BoundingBox) -> BoundingBox:
         """Create a bounding box around all other bounding boxes.
 
         :param others: one or more bounding boxes to merge with self
         :return: a bounding box around self and other bounding boxes
         :raises DeprecationWarning:
         """
-        return BoundingBox.merged(self, *others)
+        return BoundingBox.union(self, *others)
+
+    def intersect(self, *bboxes: SupportsBounds) -> BoundingBox | None:
+        """Create a bounding box around the intersection of all other bounding boxes.
+
+        :param bboxes: one or more bounding boxes to intersect with self
+        :return: a bounding box around the intersection of self and other bounding
+             boxes, or None if there is no intersection
+        """
+        return BoundingBox.intersection(self, *bboxes)
 
     @classmethod
-    def merged(cls, *bboxes: SupportsBounds) -> BoundingBox:
+    def union(cls, *bboxes: SupportsBounds) -> BoundingBox:
         """Create a bounding box around all other bounding boxes.
 
         :param bboxes: one or more bounding boxes
         :return: a bounding box encompasing all bboxes args
         :raises ValueError: if no bboxes are given
-
-        This can be used to repace a bounding box after the element it bounds has
-        been transformed with instance.transform_string.
         """
         if not bboxes:
             msg = "At least one bounding box is required"
             raise ValueError(msg)
-        min_x = min(x.x for x in bboxes)
-        max_x = max(x.x + x.width for x in bboxes)
-        min_y = min(x.y for x in bboxes)
-        max_y = max(x.y + x.height for x in bboxes)
+        xs = [*(x.x for x in bboxes), *(x.x2 for x in bboxes)]
+        ys = [*(x.y for x in bboxes), *(x.y2 for x in bboxes)]
+        min_x, max_x = min(xs), max(xs)
+        min_y, max_y = min(ys), max(ys)
         return BoundingBox(min_x, min_y, max_x - min_x, max_y - min_y)
+
+    @classmethod
+    def intersection(cls, *bboxes: SupportsBounds) -> BoundingBox | None:
+        """Create a bounding box around the intersection of all other bounding boxes.
+
+        :param bboxes: one or more bounding boxes
+        :return: a bounding box around the intersection of all bboxes, or None if
+            there is no intersection
+        :raises ValueError: if no bboxes are given
+        """
+        if not bboxes:
+            return None
+        valid_bboxes: list[BoundingBox] = []
+        for bbox in bboxes:
+            x, x2 = sorted([bbox.x, bbox.x2])
+            y, y2 = sorted([bbox.y, bbox.y2])
+            valid_bboxes.append(BoundingBox(x, y, x2 - x, y2 - y))
+        x = max(x.x for x in valid_bboxes)
+        x2 = min(x.x2 for x in valid_bboxes)
+        y = max(x.y for x in valid_bboxes)
+        y2 = min(x.y2 for x in valid_bboxes)
+        if x > x2 or y > y2:
+            return None
+        return BoundingBox(x, y, x2 - x, y2 - y)
