@@ -4,37 +4,28 @@ A text element (presumably), an svg_ultralight BoundingBox around that element,
 padding on each side of that box. This is to simplify treating scaling and moving a
 text element as if it were written on a ruled sheet of paper.
 
-Padding represents the left margin, right margin, baseline, and capline of the text.
-Baseling and capline padding will often be less than zero, as descenders and
-ascenders will extend below the baseline and above the capline.
+Padding represents the space between the direction-most point of the text and the
+left margin, right margin, descent, and ascent of the text. Top and bottom padding
+may be less than zero if the constructor used a `y_bounds_reference` argument, as
+descenders and ascenders may extend below and above the bounds of that reference
+character.
 
 There is a getter and setter for each of the four padding values. These *do not* move
 the text element. For instance, if you decrease the left padding, the left margin
 will move, *not* the text element.
 
-_There is a getter and setter for each of lmargin, rmargin, baseline, and capline.
-These *do* move the element, but do not scale it. For instance, if you move the
-leftmargin to the left, the right margin (and the text element with it) will move to
-the left.
+There is a getter and setter for each of x, cx, x2, y, cy, and y2. These *do* move
+the element, but do not scale it. For instance, if you move the left margin (x value)
+to the left, the right margin (and the text element with it) will move to the left.
 
-There is a getter and setter for padded_width and padded_height. These scale the
-element and the top and bottom padding, but *not* the left and right padding. This is
-one of two quirks which make this PaddedText class different from a generalized
-padded bounding box.
+There are getters and setters for width, height, and scale. These scale the text and
+the padding values.
 
-1. As above, the left and right padding are not scaled with the text element, the top
-and bottom padding are. This preserves but does not exaggerate the natural
-sidebearings of the text element.  This lack of scaling will be pronounced if
-adjacent padded lines are scaled to dramatically different sizes. The idea is to
-scale each PaddedText as little as possible to match widths (or any other
-relationship) then scale the resulting transformed text elements another way. For
-instance, create multiple PaddedText instances, scale their padded_width atributes to
-match, then put the resulting elements in a <g> element and scale the <g> element to
-the ultimate desired size.
-
-2. The left margin and baseline (*bottom* and left) do not move when the height or
-width is changed. This is in contrast to an InkScape rect element, which, when the
-width or height is changed, preserve the *top* and left boundaries.
+`set_width_preserve_sidebearings()`, `set_height_preserve_sidebearings(), and
+`transform_preserve_sidebearings()` methods scale the text and the top and bottom
+padding, but not the left or right padding. These also keep the text element anchored
+on `x` and `y2`. These methods are useful for aligning text of different sizes on,
+for instance, a business card so that Ls or Hs of different sizes line up vertically.
 
 Building an honest instance of this class is fairly involved:
 
@@ -51,12 +42,8 @@ Building an honest instance of this class is fairly involved:
 5. Use a BoundingBox around a "normal" capital (e.g. "M") to infer the baseline and
    capline and then calculate the top and bottom margins.
 
-There is a function to do this is `svg_ultralight.query.py` with sensible defaults.
-
-A lot can be done with a dishonest instance of this class. For instance, you could
-align and scale text while preserving left margin. The capline would scale with the
-height or width, so a left margin and capline (assume baseline is zero) would be
-enough to lay out text on a business card.
+The padded text initializers in bounding_boxes.padded_text_initializers create
+PaddedText instances with sensible defaults.
 
 :author: Shay Hill
 :created: 2021-11-28
@@ -88,6 +75,12 @@ _no_line_gap_msg = par(
     You can set an arbitrary line_gap after init with `instance.line_gap = value`."""
 )
 
+_no_font_size_msg = par(
+    """No font_size defined. Font size is an inherent font attribute defined within a
+    font file or an argument passed to `pad_text`. Any instance created with a padded
+    text initializer should have this property."""
+)
+
 
 class PaddedText(BoundElement):
     """A line of text with a bounding box and padding."""
@@ -101,6 +94,7 @@ class PaddedText(BoundElement):
         bpad: float,
         lpad: float,
         line_gap: float | None = None,
+        font_size: float | None = None,
     ) -> None:
         """Initialize a PaddedText instance.
 
@@ -110,6 +104,8 @@ class PaddedText(BoundElement):
         :param rpad: Right padding.
         :param bpad: Bottom padding.
         :param lpad: Left padding.
+        :param line_gap: The line gap between this line of text and the next. This is
+            an inherent font attribute sometimes defined within a font file.
         """
         self.elem = elem
         self.unpadded_bbox = bbox
@@ -118,6 +114,25 @@ class PaddedText(BoundElement):
         self.base_bpad = bpad
         self.lpad = lpad
         self._line_gap = line_gap
+        self._font_size = font_size
+
+    @property
+    def tbox(self) -> BoundingBox:
+        """Return the unpadded BoundingBox around the text element.
+
+        Tight bbox or True bbox. An alias for unpadded_bbox.
+
+        :return: The unpadded BoundingBox around the text element.
+        """
+        return self.unpadded_bbox
+
+    @tbox.setter
+    def tbox(self, value: BoundingBox) -> None:
+        """Set the unpadded BoundingBox around the text element.
+
+        :param value: The new unpadded BoundingBox.
+        """
+        self.unpadded_bbox = value
 
     @property
     def bbox(self) -> BoundingBox:
@@ -154,17 +169,57 @@ class PaddedText(BoundElement):
         scale: tuple[float, float] | float | None = None,
         dx: float | None = None,
         dy: float | None = None,
-    ):
+        reverse: bool = False,
+    ) -> None:
         """Transform the element and bounding box.
 
         :param transformation: a 6-tuple transformation matrix
         :param scale: a scaling factor
         :param dx: the x translation
         :param dy: the y translation
+        :param reverse: Transform the element as if it were in a <g> element
+            transformed by tmat.
         """
         tmat = new_transformation_matrix(transformation, scale=scale, dx=dx, dy=dy)
-        self.unpadded_bbox.transform(tmat)
-        _ = transform_element(self.elem, tmat)
+        self.unpadded_bbox.transform(tmat, reverse=reverse)
+        _ = transform_element(self.elem, tmat, reverse=reverse)
+        x_norm = pow(tmat[0] ** 2 + tmat[1] ** 2, 1 / 2)
+        self.lpad *= x_norm
+        self.rpad *= x_norm
+        if self._line_gap or self._font_size:
+            y_norm = pow(tmat[2] ** 2 + tmat[3] ** 2, 1 / 2)
+            if self._line_gap:
+                self._line_gap *= y_norm
+            if self._font_size:
+                self._font_size *= y_norm
+
+    def transform_preserve_sidebearings(
+        self,
+        transformation: _Matrix | None = None,
+        *,
+        scale: tuple[float, float] | float | None = None,
+        dx: float | None = None,
+        dy: float | None = None,
+        reverse: bool = False,
+    ) -> None:
+        """Transform the element and bounding box preserving sidebearings.
+
+        :param transformation: a 6-tuple transformation matrix
+        :param scale: a scaling factor
+        :param dx: the x translation
+        :param dy: the y translation
+        :param reverse: Transform the element as if it were in a <g> element
+            transformed by tmat.
+        """
+        lpad = self.lpad
+        rpad = self.rpad
+        x = self.x
+        y2 = self.y2
+        self.transform(transformation, scale=scale, dx=dx, dy=dy, reverse=reverse)
+        self.lpad = lpad
+        self.rpad = rpad
+        self.x = x
+        self.y2 = y2
 
     @property
     def line_gap(self) -> float:
@@ -184,6 +239,24 @@ class PaddedText(BoundElement):
         """
         self._line_gap = value
 
+    @property
+    def font_size(self) -> float:
+        """The font size of this line of text.
+
+        :return: The font size of this line of text.
+        """
+        if self._font_size is None:
+            raise AttributeError(_no_font_size_msg)
+        return self._font_size
+
+    @font_size.setter
+    def font_size(self, value: float) -> None:
+        """Set the font size of this line of text.
+
+        :param value: The new font size.
+        """
+        self.transform(scale=value / self.font_size)
+
     @property
     def leading(self) -> float:
         """The leading of this line of text.
@@ -198,7 +271,7 @@ class PaddedText(BoundElement):
 
         :return: The scaled top padding of this line of text.
         """
-        return self.base_tpad * self.unpadded_bbox.scale[1]
+        return self.base_tpad * self.tbox.scale[1]
 
     @tpad.setter
     def tpad(self, value: float) -> None:
@@ -206,7 +279,7 @@ class PaddedText(BoundElement):
 
         :param value: The new top padding.
         """
-        self.base_tpad = value / self.unpadded_bbox.scale[1]
+        self.base_tpad = value / self.tbox.scale[1]
 
     @property
     def bpad(self) -> float:
@@ -214,7 +287,7 @@ class PaddedText(BoundElement):
 
         :return: The scaled bottom padding of this line of text.
         """
-        return self.base_bpad * self.unpadded_bbox.scale[1]
+        return self.base_bpad * self.tbox.scale[1]
 
     @bpad.setter
     def bpad(self, value: float) -> None:
@@ -222,7 +295,7 @@ class PaddedText(BoundElement):
 
         :param value: The new bottom padding.
         """
-        self.base_bpad = value / self.unpadded_bbox.scale[1]
+        self.base_bpad = value / self.tbox.scale[1]
 
     @property
     def scale(self) -> tuple[float, float]:
@@ -237,7 +310,7 @@ class PaddedText(BoundElement):
         width*scale, height => height*scale, scale => scale*scale. This matches how
         scale works in almost every other context.
         """
-        xx, xy, yx, yy, *_ = self.unpadded_bbox.transformation
+        xx, xy, yx, yy, *_ = self.tbox.transformation
         return math.sqrt(xx * xx + xy * xy), math.sqrt(yx * yx + yy * yy)
 
     @scale.setter
@@ -254,18 +327,31 @@ class PaddedText(BoundElement):
         `scale *= 2` -> make it twice as big as it was.
         """
         new_scale = (
-            value[0] / self.unpadded_bbox.scale[0],
-            value[1] / self.unpadded_bbox.scale[1],
+            value[0] / self.tbox.scale[0],
+            value[1] / self.tbox.scale[1],
         )
         self.transform(scale=new_scale)
 
+    @property
+    def uniform_scale(self) -> float:
+        """Get uniform scale of the bounding box.
+
+        :return: uniform scale of the bounding box
+        :raises ValueError: if the scale is non-uniform.
+        """
+        scale = self.scale
+        if math.isclose(scale[0], scale[1]):
+            return scale[0]
+        msg = f"Non-uniform scale detected: sx={scale[0]}, sy={scale[1]}"
+        raise ValueError(msg)
+
     @property
     def width(self) -> float:
         """The width of this line of text with padding.
 
         :return: The scaled width of this line of text with padding.
         """
-        return self.unpadded_bbox.width + self.lpad + self.rpad
+        return self.tbox.width + self.lpad + self.rpad
 
     @width.setter
     def width(self, value: float) -> None:
@@ -280,14 +366,18 @@ class PaddedText(BoundElement):
         baseline is near y2 (y + height) not y. So, we preserve baseline (alter y
         *and* y2) when scaling.
         """
-        y2 = self.y2
+        self.transform(scale=value / self.width)
 
-        no_margins_old = self.unpadded_bbox.width
+    def set_width_preserve_sidebearings(self, value: float) -> None:
+        """Set the width of this line of text without scaling sidebearings.
+
+        :param value: The new width of this line of text.
+        :effects: the text_element bounding box is scaled to width - lpad - rpad.
+        """
+        no_margins_old = self.tbox.width
         no_margins_new = value - self.lpad - self.rpad
         scale = no_margins_new / no_margins_old
-        self.transform(scale=(scale, scale))
-
-        self.y2 = y2
+        self.transform_preserve_sidebearings(scale=scale)
 
     @property
     def height(self) -> float:
@@ -295,7 +385,7 @@ class PaddedText(BoundElement):
 
         :return: The scaled height of this line of text with padding.
         """
-        return self.unpadded_bbox.height + self.tpad + self.bpad
+        return self.tbox.height + self.tpad + self.bpad
 
     @height.setter
     def height(self, value: float) -> None:
@@ -304,10 +394,16 @@ class PaddedText(BoundElement):
         :param height: The new height of this line of text.
         :effects: the text_element bounding box is scaled to height - tpad - bpad.
         """
-        y2 = self.y2
         scale = value / self.height
-        self.transform(scale=(scale, scale))
-        self.y2 = y2
+        self.transform(scale=scale)
+
+    def set_height_preserve_sidebearings(self, value: float) -> None:
+        """Set the height of this line of text without scaling sidebearings.
+
+        :param value: The new height of this line of text.
+        :effects: the text_element bounding box is scaled to height - tpad - bpad.
+        """
+        self.transform_preserve_sidebearings(scale=value / self.height)
 
     @property
     def x(self) -> float:
@@ -315,7 +411,7 @@ class PaddedText(BoundElement):
 
         :return: The left margin of this line of text.
         """
-        return self.unpadded_bbox.x - self.lpad
+        return self.tbox.x - self.lpad
 
     @x.setter
     def x(self, value: float) -> None:
@@ -323,7 +419,7 @@ class PaddedText(BoundElement):
 
         :param value: The left margin of this line of text.
         """
-        self.transform(dx=value + self.lpad - self.unpadded_bbox.x)
+        self.transform(dx=value + self.lpad - self.tbox.x)
 
     @property
     def cx(self) -> float:
@@ -347,7 +443,7 @@ class PaddedText(BoundElement):
 
         :return: The right margin of this line of text.
         """
-        return self.unpadded_bbox.x2 + self.rpad
+        return self.tbox.x2 + self.rpad
 
     @x2.setter
     def x2(self, value: float) -> None:
@@ -355,7 +451,7 @@ class PaddedText(BoundElement):
 
         :param value: The right margin of this line of text.
         """
-        self.transform(dx=value - self.rpad - self.unpadded_bbox.x2)
+        self.transform(dx=value - self.rpad - self.tbox.x2)
 
     @property
     def y(self) -> float:
@@ -363,7 +459,7 @@ class PaddedText(BoundElement):
 
         :return: The top of this line of text.
         """
-        return self.unpadded_bbox.y - self.tpad
+        return self.tbox.y - self.tpad
 
     @y.setter
     def y(self, value: float) -> None:
@@ -371,7 +467,7 @@ class PaddedText(BoundElement):
 
         :param value: The top of this line of text.
         """
-        self.transform(dy=value + self.tpad - self.unpadded_bbox.y)
+        self.transform(dy=value + self.tpad - self.tbox.y)
 
     @property
     def cy(self) -> float:
@@ -395,7 +491,7 @@ class PaddedText(BoundElement):
 
         :return: The bottom of this line of text.
         """
-        return self.unpadded_bbox.y2 + self.bpad
+        return self.tbox.y2 + self.bpad
 
     @y2.setter
     def y2(self, value: float) -> None:
@@ -403,9 +499,4 @@ class PaddedText(BoundElement):
 
         :param value: The bottom of this line of text.
         """
-        self.transform(dy=value - self.bpad - self.unpadded_bbox.y2)
-
-    lmargin = x
-    rmargin = x2
-    capline = y
-    baseline = y2
+        self.transform(dy=value - self.bpad - self.tbox.y2)
