@@ -109,9 +109,12 @@ from paragraphs import par
 from svg_path_data import format_svgd_shortest, get_cpts_from_svgd, get_svgd_from_cpts
 
 from svg_ultralight.bounding_boxes.type_bounding_box import BoundingBox
-from svg_ultralight.constructors.new_element import new_element
-from svg_ultralight.font_tools.globs import DEFAULT_FONT_SIZE
-from svg_ultralight.string_conversion import format_numbers
+from svg_ultralight.constructors.new_element import (
+    new_element,
+    new_sub_element,
+    update_element,
+)
+from svg_ultralight.strings import svg_matrix
 
 if TYPE_CHECKING:
     import os
@@ -124,7 +127,15 @@ if TYPE_CHECKING:
 logging.getLogger("fontTools").setLevel(logging.ERROR)
 
 
-_ESCAPE_CHARS = {"&": "&amp;", "<": "&lt;", ">": "&gt;", '"': "&quot;", "'": "&apos;"}
+_ESCAPE_CHARS = {
+    "&": "&amp;",
+    "<": "&lt;",
+    ">": "&gt;",
+    '"': "&quot;",
+    "'": "&apos;",
+    "{": "&#123",  # valid, but stops MS File Explorer from thumbnailing an svg
+    "}": "&#125",  # valid, but stops MS File Explorer from thumbnailing an svg
+}
 
 
 def _sanitize_svg_data_text(text: str) -> str:
@@ -138,8 +149,6 @@ def _sanitize_svg_data_text(text: str) -> str:
     return text
 
 
-# extract_gpos_kerning is an unfinished attempt to extract kerning from the GPOS
-# table.
 def _get_gpos_kerning(font: TTFont) -> dict[tuple[str, str], int]:
     """Extract kerning pairs from the GPOS table of a font.
 
@@ -161,7 +170,9 @@ def _get_gpos_kerning(font: TTFont) -> dict[tuple[str, str], int]:
     type2_lookups = (x for x in gpos.LookupList.Lookup if x.LookupType == 2)
     subtables = list(it.chain(*(x.SubTable for x in type2_lookups)))
     for subtable in (x for x in subtables if x.Format == 1):  # glyph-pair kerning
-        for pair_set, glyph1 in zip(subtable.PairSet, subtable.Coverage.glyphs):
+        for pair_set, glyph1 in zip(
+            subtable.PairSet, subtable.Coverage.glyphs, strict=True
+        ):
             for pair_value in pair_set.PairValueRecord:
                 glyph2 = pair_value.SecondGlyph
                 value1 = pair_value.Value1
@@ -214,7 +225,7 @@ def _split_into_quadratic(*pts: _XYTuple) -> Iterator[tuple[_XYTuple, _XYTuple]]
         msg = "At least two points are required."
         raise ValueError(msg)
     for prev_cp, next_cp in it.pairwise(pts[:-1]):
-        xs, ys = zip(prev_cp, next_cp)
+        xs, ys = zip(prev_cp, next_cp, strict=True)
         midpnt = sum(xs) / 2, sum(ys) / 2
         yield prev_cp, midpnt
     yield pts[-2], pts[-1]
@@ -236,7 +247,7 @@ class PathPen(BasePen):
         """Return an svg path data string for the glyph."""
         if not self._cmds:
             return ""
-        svgd = format_svgd_shortest(" ".join(self._cmds))
+        svgd = " ".join(self._cmds)
         return "M" + svgd[1:]
 
     @property
@@ -270,7 +281,7 @@ class PathPen(BasePen):
         for q_pts in _split_into_quadratic(*pts):
             self._cmds.extend(("Q", *map(str, it.chain(*q_pts))))
 
-    def closePath(self):
+    def closePath(self) -> None:
         """Close the current path."""
         self._cmds.append("Z")
 
@@ -377,13 +388,13 @@ class FTFontInfo:
         :param dx: An optional x translation to apply to the glyph.
         :return: The svg path data for the character.
         """
+        glyph_name = self.get_glyph_name(char)
         glyph_set = self.font.getGlyphSet()
-        glyph_name = self.font.getBestCmap().get(ord(char))
         path_pen = PathPen(glyph_set)
         _ = glyph_set[glyph_name].draw(path_pen)
         svgd = path_pen.svgd
         if not dx or not svgd:
-            return svgd
+            return format_svgd_shortest(svgd)
         cpts = get_cpts_from_svgd(svgd)
         for i, curve in enumerate(cpts):
             cpts[i][:] = [(x + dx, y) for x, y in curve]
@@ -398,15 +409,15 @@ class FTFontInfo:
         same, but when they disagree, this method is more accurate. Additionally,
         some fonts do not have a glyf table, so this method is more robust.
         """
+        glyph_name = self.get_glyph_name(char)
         glyph_set = self.font.getGlyphSet()
-        glyph_name = self.font.getBestCmap().get(ord(char))
         bounds_pen = BoundsPen(glyph_set)
         _ = glyph_set[glyph_name].draw(bounds_pen)
         pen_bounds = cast("None | tuple[int, int, int, int]", bounds_pen.bounds)
         if pen_bounds is None:
             return 0, 0, 0, 0
-        xMin, yMin, xMax, yMax = pen_bounds
-        return xMin, yMin, xMax, yMax
+        x_min, y_min, x_max, y_max = pen_bounds
+        return x_min, y_min, x_max, y_max
 
     def get_char_bbox(self, char: str) -> BoundingBox:
         """Return the BoundingBox of a character svg coordinates.
@@ -440,7 +451,7 @@ class FTFontInfo:
         bounds = [self.get_char_bounds(c) for c in text]
         total_advance = sum(hmtx[n][0] for n in names[:-1])
         total_kern = sum(self.kern_table.get((x, y), 0) for x, y in it.pairwise(names))
-        min_xs, min_ys, max_xs, max_ys = zip(*bounds)
+        min_xs, min_ys, max_xs, max_ys = zip(*bounds, strict=True)
         min_x = min_xs[0]
         min_y = min(min_ys)
 
@@ -448,6 +459,23 @@ class FTFontInfo:
         max_y = max(max_ys)
         return min_x, min_y, max_x, max_y
 
+    def get_text_svgd_by_char(self, text: str, dx: float = 0) -> Iterator[str]:
+        """Return an iterator of svg path data for each character in a string.
+
+        :param text: The text to get the svg path data for.
+        :param dx: An optional x translation to apply to the entire text.
+        :return: An iterator of svg path data for each character in the text.
+        """
+        hmtx = cast("dict[str, tuple[int, int]]", self.font["hmtx"])
+        char_dx = dx
+        for c_this, c_next in it.pairwise(text):
+            this_name = self.get_glyph_name(c_this)
+            next_name = self.get_glyph_name(c_next)
+            yield self.get_char_svgd(c_this, char_dx)
+            char_dx += hmtx[this_name][0]
+            char_dx += self.kern_table.get((this_name, next_name), 0)
+        yield self.get_char_svgd(text[-1], char_dx)
+
     def get_text_svgd(self, text: str, dx: float = 0) -> str:
         """Return the svg path data for a string.
 
@@ -455,17 +483,29 @@ class FTFontInfo:
         :param dx: An optional x translation to apply to the entire text.
         :return: The svg path data for the text.
         """
+        return "".join(self.get_text_svgd_by_char(text, dx))
+
+    def get_text_svgds(self, text: str, dx: float = 0) -> list[tuple[str, str]]:
+        """Return a list of svg path data for each character in a string.
+
+        :param text: The text to get the svg path data for.
+        :param dx: An optional x translation to apply to the entire text.
+        :return: A list of svg path data for each character in the text.
+        """
         hmtx = cast("dict[str, tuple[int, int]]", self.font["hmtx"])
-        svgd = ""
-        char_dx = dx
+        svgds: list[tuple[str, str]] = []
         for c_this, c_next in it.pairwise(text):
             this_name = self.get_glyph_name(c_this)
             next_name = self.get_glyph_name(c_next)
-            svgd += self.get_char_svgd(c_this, char_dx)
-            char_dx += hmtx[this_name][0]
-            char_dx += self.kern_table.get((this_name, next_name), 0)
-        svgd += self.get_char_svgd(text[-1], char_dx)
-        return svgd
+            svgd = self.get_char_svgd(c_this)
+            tmat = "" if dx == 0 else f"matrix(1 0 0 1 {dx} 0)"
+            svgds.append((svgd, tmat))
+            dx += hmtx[this_name][0]
+            dx += self.kern_table.get((this_name, next_name), 0)
+        svgd = self.get_char_svgd(text[-1])
+        tmat = "" if dx == 0 else f"matrix(1 0 0 1 {dx} 0)"
+        svgds.append((svgd, tmat))
+        return svgds
 
     def get_text_bbox(self, text: str) -> BoundingBox:
         """Return the BoundingBox of a string svg coordinates.
@@ -499,7 +539,7 @@ class FTTextInfo:
         self,
         font: str | os.PathLike[str] | FTFontInfo,
         text: str,
-        font_size: float,
+        font_size: float | None = None,
         ascent: float | None = None,
         descent: float | None = None,
     ) -> None:
@@ -509,7 +549,7 @@ class FTTextInfo:
         else:
             self._font = FTFontInfo(font)
         self._text = text.rstrip(" ")
-        self._font_size = font_size
+        self._font_size = font_size or self._font.units_per_em
         self._ascent = ascent
         self._descent = descent
 
@@ -539,17 +579,38 @@ class FTTextInfo:
     def new_element(self, **attributes: ElemAttrib) -> EtreeElement:
         """Return an svg text element with the appropriate font attributes."""
         matrix_vals = (self.scale, 0, 0, -self.scale, 0, 0)
-        matrix = f"matrix({' '.join(format_numbers(matrix_vals))})"
-        attributes["transform"] = matrix
+        attributes["transform"] = svg_matrix(matrix_vals)
         stroke_width = attributes.get("stroke-width")
         if stroke_width:
             attributes["stroke-width"] = float(stroke_width) / self.scale
-        return new_element(
-            "path",
-            data_text=_sanitize_svg_data_text(self.text),
-            d=self.font.get_text_svgd(self.text),
-            **attributes,
-        )
+        data_text = _sanitize_svg_data_text(self.text)
+        group = new_element("g", data_text=data_text, **attributes)
+        for i, (svgd, tmat) in enumerate(self.font.get_text_svgds(self.text)):
+            if not svgd:
+                continue
+            data_text = _sanitize_svg_data_text(self.text[i])
+            if not tmat:
+                _ = new_sub_element(group, "path", data_text=data_text, d=svgd)
+                continue
+            _ = new_sub_element(
+                group, "path", data_text=data_text, d=svgd, transform=tmat
+            )
+        return group
+
+    def new_chars_group_element(self, **attributes: ElemAttrib) -> EtreeElement:
+        """Return an svg group element with a path for each character in the text."""
+        names = map(self.font.get_glyph_name, self.text)
+        svgds = self.font.get_text_svgd_by_char(self.text)
+        group = new_element("g", **attributes)
+        for name, svgd in zip(names, svgds, strict=True):
+            data_text = _sanitize_svg_data_text(name)
+            _ = new_sub_element(group, "path", data_text=data_text, d=svgd)
+        matrix_vals = (self.scale, 0, 0, -self.scale, 0, 0)
+        group.attrib["transform"] = svg_matrix(matrix_vals)
+        stroke_width = group.attrib.get("stroke-width")
+        if stroke_width:
+            _ = update_element(group, stroke_width=float(stroke_width) / self.scale)
+        return group
 
     @property
     def bbox(self) -> BoundingBox:
@@ -636,9 +697,9 @@ def get_font_size_given_height(font: str | os.PathLike[str], height: float) -> f
 
 
 def get_padded_text_info(
-    font: str | os.PathLike[str],
+    font: str | os.PathLike[str] | FTFontInfo,
     text: str,
-    font_size: float = DEFAULT_FONT_SIZE,
+    font_size: float | None = None,
     ascent: float | None = None,
     descent: float | None = None,
     *,
@@ -661,7 +722,10 @@ def get_padded_text_info(
     :return: A FTTextInfo object with the information necessary to create a
         PaddedText instance: bbox, tpad, rpad, bpad, lpad.
     """
-    font_info = FTFontInfo(font)
+    if isinstance(font, FTFontInfo):
+        font_info = font
+    else:
+        font_info = FTFontInfo(font)
     if y_bounds_reference:
         capline_info = FTTextInfo(font_info, y_bounds_reference, font_size)
         ascent = -capline_info.bbox.y
