@@ -14,6 +14,8 @@ from __future__ import annotations
 import dataclasses
 import enum
 import re
+from contextlib import suppress
+from typing import Any, Literal, TypeAlias, TypeGuard, cast
 
 from svg_ultralight.string_conversion import format_number
 
@@ -47,16 +49,10 @@ class Unit(enum.Enum):
     USER = "", 1  # "user units" without a unit specifier
 
 
-# the arguments this module will attempt to interpret as a string with a unit specifier
-MeasurementArg = (
-    float
-    | str
-    | tuple[str, str]
-    | tuple[float, str]
-    | tuple[str, Unit]
-    | tuple[float, Unit]
-    | Unit
-)
+_UnitSpecifier: TypeAlias = Literal[
+    "in", "pt", "px", "mm", "cm", "m", "km", "Q", "pc", "yd", "ft", ""
+]
+
 
 _UNIT_SPECIFIER2UNIT = {x.value[0]: x for x in Unit}
 
@@ -67,6 +63,28 @@ _NUMBER_RE = re.compile(rf"(?P<number>{_NUMBER})")
 _NUMBER_AND_UNIT = re.compile(rf"^{_NUMBER_RE.pattern}{_UNIT_RE.pattern}$")
 
 
+def is_unit_specifier(obj: object) -> TypeGuard[_UnitSpecifier]:
+    """Determine if an object is a valid unit specifier.
+
+    :param obj: object to check
+    :return: True if the object is a valid unit specifier
+    """
+    return isinstance(obj, str) and obj in _UNIT_SPECIFIER2UNIT
+
+
+def is_measurement_arg(obj: object) -> TypeGuard[MeasurementArg]:
+    """Determine if an object is a valid measurement argument.
+
+    :param obj: object to check
+    :return: True if the object is a valid measurement argument
+    """
+    maybe_measurement_arg = cast("Any", obj)
+    with suppress(ValueError):
+        _ = Measurement(maybe_measurement_arg)
+        return True
+    return False
+
+
 def _parse_unit(measurement_arg: MeasurementArg) -> tuple[float, Unit]:
     """Split the value and unit from a string.
 
@@ -86,7 +104,6 @@ def _parse_unit(measurement_arg: MeasurementArg) -> tuple[float, Unit]:
     | float         | 55.32              | (55.32, Unit.USER) |
     | str           | "55.32px"          | (55.32, Unit.PX)   |
     | str           | "55.32"            | (55.32, Unit.USER) |
-    | str           | "px"               | (0.0, Unit.PX)     |
     | (str, str)    | ("55.32", "px")    | (55.32, Unit.PX)   |
     | (float, str)  | (55.32, "px")      | (55.32, Unit.PX)   |
     | (str, Unit)   | ("55.32", Unit.PX) | (55.32, Unit.PX)   |
@@ -95,8 +112,10 @@ def _parse_unit(measurement_arg: MeasurementArg) -> tuple[float, Unit]:
     | Measurement   | Measurement("3in") | (3.0, Unit.IN)     |
 
     """
+    if isinstance(measurement_arg, Measurement):
+        return measurement_arg.get_tuple()
     failure_msg = f"Cannot parse value and unit from {measurement_arg}"
-    unit: str | Unit
+    unit: _UnitSpecifier | Unit
     try:
         if isinstance(measurement_arg, tuple):
             number, unit = float(measurement_arg[0]), measurement_arg[1]
@@ -111,12 +130,10 @@ def _parse_unit(measurement_arg: MeasurementArg) -> tuple[float, Unit]:
             return _parse_unit((0, measurement_arg))
 
         if number_unit := _NUMBER_AND_UNIT.match(str(measurement_arg)):
-            return _parse_unit((number_unit["number"], number_unit["unit"]))
+            unit = _UNIT_SPECIFIER2UNIT[number_unit["unit"]]
+            return _parse_unit((number_unit["number"], unit))
 
-        if unit_only := _UNIT_RE.match(str(measurement_arg)):
-            return _parse_unit((0, unit_only["unit"]))
-
-    except (ValueError, KeyError) as e:
+    except (ValueError, KeyError, IndexError) as e:
         raise ValueError(failure_msg) from e
 
     raise ValueError(failure_msg)
@@ -175,7 +192,7 @@ class Measurement:
         single measurements. Single measurements can be defined by something like
         `(1, "in")`, but groups can be passed as single or tuples, so there is no way
         to differentiate between (1, "in") and "1in" or (1, "in") as ("1", "0in").
-        That is a limitation, but doint it that way preserved the flexibility (and
+        That is a limitation, but doing it that way preserved the flexibility (and
         backwards compatibility) of being able to define padding as "1in" everywhere
         or (1, 2, 3, 4) for top, right, bottom, left.
 
@@ -190,13 +207,13 @@ class Measurement:
     def get_svg(self, unit: Unit | None = None) -> str:
         """Get the measurement in the specified unit as it would be written in svg.
 
-        :param optional unit: the unit to convert to
+        :param optional unit: the unit to convert to (defaults to native unit)
         :return: the measurement in the specified unit, always as a string
 
         Rounds values to 6 decimal places as recommended by svg guidance online.
-        Higher precision just changes file size without imroving quality.
+        Higher resolution just changes file size without imroving quality.
         """
-        _, unit = self.get_tuple(unit)
+        _, unit = self.get_tuple(unit or self.native_unit)
         value_as_str = format_number(self.get_value(unit))
         return f"{value_as_str}{unit.value[0]}"
 
@@ -245,3 +262,36 @@ class Measurement:
         :return: the measurement divided by the scalar in self native unit
         """
         return self.__mul__(1.0 / scalar)
+
+
+def to_user_units(measurement_arg: MeasurementArg) -> float:
+    """Convert a measurement argument to user units.
+
+    :param measurement_arg: The measurement argument to convert
+    :return: The measurement in user units
+    """
+    if isinstance(measurement_arg, (int, float)):
+        return float(measurement_arg)
+    return Measurement(measurement_arg).value
+
+
+def to_svg_str(measurement_arg: MeasurementArg) -> str:
+    """Convert a measurement argument to an svg string.
+
+    :param measurement_arg: The measurement argument to convert
+    :return: The measurement as an svg string
+    """
+    return Measurement(measurement_arg).get_svg()
+
+
+# the arguments this module will attempt to interpret as a string with a unit specifier
+MeasurementArg: TypeAlias = (
+    float
+    | str
+    | tuple[str, _UnitSpecifier]
+    | tuple[float, _UnitSpecifier]
+    | tuple[str, Unit]
+    | tuple[float, Unit]
+    | Unit
+    | Measurement
+)
